ROOT="/"
FIRMWARE_INSTALL_PATH="#{ROOT}/protonet/firmware/"
CONFIG_BASE="#{ROOT}/data/var/lib/protonet/config/"

require 'digest/md5'
require 'fileutils'
require 'open-uri'
require 'json'
require File.dirname(__FILE__) + '/protonet_config'
require File.dirname(__FILE__) + '/version'

class FirmwareUpdater
  def initialize(version)
    @version = version
  end

  def exists?(md5, path)
    (Digest::MD5.file(path).hexdigest == md5) rescue false
  end

  def download_and_check(md5, path)
    puts "downloading #{path}"
    FileUtils.mkdir_p(File.dirname(path))
    `curl -s #{FIRMWARE_BASE_URL}/#{path} > .download_temp && mv .download_temp #{path}`
    raise unless exists?(md5, path)
  end

  def load_manifest
    image_files = open("#{FIRMWARE_BASE_URL}/#{@version}/MANIFEST").
      read.split("\n").
      map { |line| line.split(/\s+/) }
  end

  def ensure_downloaded
    puts "updating #{@version}"

    firmware_files = load_manifest

    firmware_files.each do |md5, path|
      next if exists?(md5, path)
      download_and_check(md5, path)
    end

    files_not_in_firmware = Dir["#{@version}/**/**"].reject { |path| File.directory?(path) }.sort - firmware_files.map { |i| i[1] }.sort

    if files_not_in_firmware.count > 0
      FileUtils.rm(files_not_in_firmware)      
      puts "deleted #{files_not_in_firmware.join(", ")}"
    end

    FileUtils.ln(IMAGE, "#{@version}/image")

    true # all good!
  end

  def activate_firmware
    puts "activating #{@version}"

    FileUtils.rm_f('current')
    FileUtils.ln_s("./#{@version}", "current")

    # TODO: only if new kernel
    begin
      `dpkg --force-all -i #{KERNEL}` 
    rescue Exception => e
      puts "ERROR: Cannot install kernel! #{e.inspect}"
    end
  end
end

def generate_protonet_config
  config = protonet_config
  puts "generating config"
  File.open("#{CONFIG_BASE}/current", "w") { |f| f.write JSON.pretty_generate(config) }
  FileUtils.chmod 0664, "#{CONFIG_BASE}/current"
  FileUtils.chown 'protonet', 'protonet', "#{CONFIG_BASE}/current"
  true
rescue Exception => e
  puts "ERROR: cannot generate config! #{e.inspect}"
  false
end

# def protonet_config_exists?
#   JSON.load("#{CONFIG_BASE}/current") rescue false
# end

# unless protonet_config_exists?
#   generate_protonet_config
# end

def prep_system
  FileUtils.mkdir_p(FIRMWARE_INSTALL_PATH)
  FileUtils.mkdir_p(CONFIG_BASE)

  Dir.chdir(ROOT) do 
    FileUtils.rm_f("firmware")
    FileUtils.ln_s("./protonet/firmware", "firmware")
  end

  true
end

def download_and_activate_firmware(version)
  Dir.chdir(FIRMWARE_INSTALL_PATH) do 
    updater = FirmwareUpdater.new(version)
    updater.ensure_downloaded && updater.activate_firmware
  end

  true
end

def update_firmware(version)
  Dir.chdir(FIRMWARE_INSTALL_PATH) do 
    updater = FirmwareUpdater.new(version)
    updater.ensure_downloaded
  end
end

def migrate_protonet_data
  return unless File.directory?('/home/protonet/dashboard')
  puts "migrate_protonet_data"

  `sv down /home/protonet/dashboard/shared/services/enabled/*`

  `mkdir -p /data/home/protonet/dashboard/shared`
  `mv /home/protonet/dashboard/shared/files /data/home/protonet/dashboard/shared`
  `mv /home/protonet/dashboard/shared/public /data/home/protonet/dashboard/shared`
  `mv /home/protonet/dashboard/shared/log /data/home/protonet/dashboard/shared`
  `mkdir -p /data/var/lib`
  `mv /var/lib/mysql /data/var/lib`
  `mv /home/protonet/dashboard /home/protonet/inactive_dashboard`
  `mv /etc/sv/protonet_runit /etc/protonet_runit.inactive`
end

def undo_migrate_protonet_data
  return unless File.directory?('/home/protonet/inactive_dashboard')
  puts "undo_migrate_protonet_data"

  `mv /data/home/protonet/dashboard/shared/files /home/protonet/dashboard/shared/`
  `mv /data/home/protonet/dashboard/shared/public /home/protonet/dashboard/shared/`
  `mv /data/home/protonet/dashboard/shared/log /home/protonet/dashboard/shared/`
  `mv /data/var/lib/mysql /var/lib`
  `mv /home/protonet/inactive_dashboard /home/protonet/dashboard`
  `mv /etc/protonet_runit.inactive /etc/sv/protonet_runit`
end

version = PROTONET_VERSION

if ARGV[0] == "update"
  update_firmware(version)
elsif ARGV[0] == "install"
  if File.exists?('/protonet/firmware/current')
    download_and_activate_firmware(version) && exit(0)
  else
    # upgrade legacy system
    prep_system && generate_protonet_config && download_and_activate_firmware(version) && migrate_protonet_data && exit(0)
  end
end

exit(1)